==> openttd-0.1.4-source.zip: openttd-0.1.4/airport_gui.c <==
#include "stdafx.h"
#include "ttd.h"

#include "window.h"
#include "gui.h"
#include "viewport.h"
#include "gfx.h"
#include "command.h"
#include "vehicle.h"
#include "station.h"

static byte _selected_airport_type;

void ShowBuildAirportPicker();

void PlaceAirport(uint tile)
{
	int32 ret;
	
	ret = DoCommandByTile(tile, _selected_airport_type, 0, 
		DC_EXEC | DC_AUTO | DC_NO_WATER | DC_MSG(STR_A001_CAN_T_BUILD_AIRPORT_HERE),
		CMD_BUILD_AIRPORT);
	
	if (ret == CMD_ERROR)
		return;
	
	SndPlayTileFx(0x1D, tile);
	ResetObjectToPlace();
}

void BuildAirClick_Airport(Window *w)
{
	SndPlayFx(0x13);
	SetWindowDirty(w);
	if (w->click_state & (1<<2)) {
		ResetObjectToPlace();
		return;
	}

	SetObjectToPlaceWnd(0xAA4, 1, w);

	w->click_state |= 1<<2;
	_place_proc = PlaceAirport;

	ShowBuildAirportPicker();
}


void BuildAirClick_Demolish(Window *w)
{
	extern const uint16 _demolish_animcursor[];

	SndPlayFx(0x13);
	SetWindowDirty(w);
	if (w->click_state & (1<<3)) {
		ResetObjectToPlace();
		return;
	}
	SetObjectToPlaceWnd((uint32)_demolish_animcursor, 1|SOTP_ANIM_CURSOR,w);
	w->click_state |= (1<<3);
	_place_proc = PlaceProc_Demolish;
}

void BuildAirClick_Raise(Window *w)
{
	extern const uint16 _raise_land_animcursor[];
	
	SndPlayFx(0x13);
	SetWindowDirty(w);

	if (w->click_state & (1<<5)) {
		ResetObjectToPlace();
		return;
	}
	
	SetObjectToPlaceWnd((uint32)_raise_land_animcursor, 2|SOTP_ANIM_CURSOR, w);

	w->click_state |= (1<<5);
	_place_proc = PlaceProc_RaiseLand;
}

void BuildAirClick_Lower(Window *w)
{
	extern const uint16 _lower_land_animcursor[];
	
	SndPlayFx(0x13);
	SetWindowDirty(w);

	if (w->click_state & (1<<4)) {
		ResetObjectToPlace();
		return;
	}

	SetObjectToPlaceWnd((uint32)_lower_land_animcursor, 2|SOTP_ANIM_CURSOR, w);

	w->click_state |= (1<<4);
	_place_proc = PlaceProc_LowerLand;
}

void BuildAirClick_Purchase(Window *w)
{
	SndPlayFx(0x13);
	SetWindowDirty(w);

	if (w->click_state & (1<<6)) {
		ResetObjectToPlace();
		return;
	}

	SetObjectToPlaceWnd(0x12B8, 1, w);

	w->click_state |= (1<<6);
	_place_proc = PlaceProc_BuyLand;
}

typedef void OnButtonClick(Window *w);
static OnButtonClick * const _build_air_button_proc[] = {
	BuildAirClick_Airport,
	BuildAirClick_Demolish,
	BuildAirClick_Lower,
	BuildAirClick_Raise,
	BuildAirClick_Purchase,
};

static void BuildAirToolbWndProc(Window *w, WindowEvent *e)
{
	switch(e->event) {
	case WE_PAINT:
		DrawWindowWidgets(w);
		break;

	case WE_CLICK:
		if (e->click.widget-2 >= 0)
			_build_air_button_proc[e->click.widget - 2](w);		
		break;

	case WE_PLACE_OBJ:
		_place_proc(e->place.tile);
		break;	

	case WE_ABORT_PLACE_OBJ:
		w->click_state = 0;
		SetWindowDirty(w);
		w = FindWindowById(WC_BUILD_STATION, 0);
		if (w != 0)
			WP(w,def_d).close = true;
		break;
	}
}

static const Widget _air_toolbar_widgets[] = {
{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,										STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,     7,    11,   129,     0,    13, STR_A000_AIRPORT_CONSTRUCT,	STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,     7,     0,    41,    14,    35, 0x2E8,												STR_A01E_BUILD_AIRPORT},
{      WWT_PANEL,     7,    42,    63,    14,    35, 0x2BF,												STR_018D_DEMOLISH_BUILDINGS_ETC},
{      WWT_PANEL,     7,    64,    85,    14,    35, 0x2B7,												STR_018E_LOWER_A_CORNER_OF_LAND},
{      WWT_PANEL,     7,    86,   107,    14,    35, 0x2B6,												STR_018F_RAISE_A_CORNER_OF_LAND},
{      WWT_PANEL,     7,   108,   129,    14,    35, 0x12B7,											STR_0329_PURCHASE_LAND_FOR_FUTURE},
{      WWT_LAST},
};


static const WindowDesc _air_toolbar_desc = {
	0x1FE, 0x16, 0x82, 0x24,
	WC_BUILD_TOOLBAR,0,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
	_air_toolbar_widgets,
	BuildAirToolbWndProc
};


void ShowBuildAirToolbar()
{
	DeleteWindowById(WC_BUILD_TOOLBAR, 0);
	AllocateWindowDescFront(&_air_toolbar_desc, 0);
}


static void BuildAirportPickerWndProc(Window *w, WindowEvent *e)
{
	switch(e->event) {
	case WE_PAINT: {
		if (WP(w,def_d).close)
			return;
		w->disabled_state = 0;

/*		if (!(_avail_aircraft & 1)) {
			w->disabled_state |= (1<<3);
			if (_selected_airport_type == AT_SMALL)
				_selected_airport_type = AT_LARGE;
		}

		if (!(_avail_aircraft & 2)) {
			w->disabled_state |= (1<<4);
			if (_selected_airport_type == AT_LARGE)
				_selected_airport_type = AT_SMALL;
		}
*/
		if (!(_avail_aircraft & 4)) {
			w->disabled_state |= (1<<5);
		}

		w->click_state = ((1<<3) << _selected_airport_type) | ((1<<6) << _station_show_coverage);

		SetTileSelectSize(_airport_size_x[_selected_airport_type],_airport_size_y[_selected_airport_type]);
		if (_station_show_coverage)	SetTileSelectBigSize(-4, -4, 8, 8);

		DrawWindowWidgets(w);

		DrawStringCentered(74, 16, STR_305B_SIZE, 0);
		DrawStringCentered(74, 53, STR_3066_COVERAGE_AREA_HIGHLIGHT, 0);

		DrawStationCoverageAreaText(2, 77, (uint)-1);
		break;
	}

	case WE_CLICK: {
		switch(e->click.widget) {
		case 0:
			ResetObjectToPlace();
			break;
		case 3: case 4: case 5:
			_selected_airport_type = e->click.widget - 3;
			SndPlayFx(0x13);
			SetWindowDirty(w);
			break;
		case 6:
		case 7:
			_station_show_coverage = e->click.widget - 6;
			SndPlayFx(0x13);
			SetWindowDirty(w);
			break;
		}
	} break;

	case WE_MOUSELOOP: {

		if (WP(w,def_d).close) {
			DeleteWindow(w);
			return;
		}

		CheckRedrawStationCoverage(w);
	} break;
	}
}

static const Widget _build_airport_picker_widgets[] = {
{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,										STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,     7,    11,   147,     0,    13, STR_3001_AIRPORT_SELECTION,	STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,     7,     0,   147,    14,   107, 0x0,													0},
{WWT_NODISTXTBTN,    14,     2,    73,    27,    38, STR_3059_SMALL,							STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{WWT_NODISTXTBTN,    14,    74,   145,    27,    38, STR_305A_LARGE,							STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{WWT_NODISTXTBTN,    14,     2,   145,    39,    50, STR_306B_HELIPORT,						STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{   WWT_CLOSEBOX,    14,    14,    73,    62,    73, STR_02DB_OFF,								STR_3065_DON_T_HIGHLIGHT_COVERAGE},
{   WWT_CLOSEBOX,    14,    74,   133,    62,    73, STR_02DA_ON,									STR_3064_HIGHLIGHT_COVERAGE_AREA},
{      WWT_LAST},
};

static const WindowDesc _build_airport_desc = {
	-1, -1, 0x94, 0x6C,
	WC_BUILD_STATION,WC_BUILD_TOOLBAR,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
	_build_airport_picker_widgets,
	BuildAirportPickerWndProc
};

void ShowBuildAirportPicker()
{
	AllocateWindowDesc(&_build_airport_desc);
}

void InitializeAirportGui()
{
	_selected_airport_type = AT_SMALL;
	_last_built_aircraft_depot_tile = 0;
}

==> openttd-0.2.1-source.zip: openttd-0.2.1/airport_gui.c <==
#include "stdafx.h"
#include "ttd.h"

#include "window.h"
#include "gui.h"
#include "viewport.h"
#include "gfx.h"
#include "command.h"
#include "vehicle.h"
#include "station.h"

static byte _selected_airport_type;

static void ShowBuildAirportPicker();

static void PlaceAirport(uint tile)
{
	int32 ret;
	
	ret = DoCommandByTile(tile, _selected_airport_type, 0, 
		DC_EXEC | DC_AUTO | DC_NO_WATER | DC_MSG(STR_A001_CAN_T_BUILD_AIRPORT_HERE),
		CMD_BUILD_AIRPORT);
	
	if (ret == CMD_ERROR)
		return;
	
	SndPlayTileFx(0x1D, tile);
	ResetObjectToPlace();
}

static void BuildAirClick_Airport(Window *w)
{
	if (HandlePlacePushButton(w, 2, 0xAA4, 1, PlaceAirport)) ShowBuildAirportPicker();
}

static void BuildAirClick_Demolish(Window *w)
{
	HandlePlacePushButton(w, 3, ANIMCURSOR_DEMOLISH, 1, PlaceProc_Demolish);
}

static void BuildAirClick_Lower(Window *w)
{
	HandlePlacePushButton(w, 4, ANIMCURSOR_LOWERLAND, 2, PlaceProc_LowerLand);
}

static void BuildAirClick_Raise(Window *w)
{
	HandlePlacePushButton(w, 5, ANIMCURSOR_RAISELAND, 2, PlaceProc_RaiseLand);
}

static void BuildAirClick_Purchase(Window *w)
{
	HandlePlacePushButton(w, 6, 0x12B8, 1, PlaceProc_BuyLand);
}

typedef void OnButtonClick(Window *w);
static OnButtonClick * const _build_air_button_proc[] = {
	BuildAirClick_Airport,
	BuildAirClick_Demolish,
	BuildAirClick_Lower,
	BuildAirClick_Raise,
	BuildAirClick_Purchase,
};

static void BuildAirToolbWndProc(Window *w, WindowEvent *e)
{
	switch(e->event) {
	case WE_PAINT:
		DrawWindowWidgets(w);
		break;

	case WE_CLICK:
		if (e->click.widget-2 >= 0)
			_build_air_button_proc[e->click.widget - 2](w);		
		break;

	case WE_PLACE_OBJ:
		_place_proc(e->place.tile);
		break;	

	case WE_ABORT_PLACE_OBJ:
		w->click_state = 0;
		SetWindowDirty(w);
		w = FindWindowById(WC_BUILD_STATION, 0);
		if (w != 0)
			WP(w,def_d).close = true;
		break;
	}
}

static const Widget _air_toolbar_widgets[] = {
{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,										STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,     7,    11,   129,     0,    13, STR_A000_AIRPORT_CONSTRUCT,	STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,     7,     0,    41,    14,    35, 0x2E8,												STR_A01E_BUILD_AIRPORT},
{      WWT_PANEL,     7,    42,    63,    14,    35, 0x2BF,												STR_018D_DEMOLISH_BUILDINGS_ETC},
{      WWT_PANEL,     7,    64,    85,    14,    35, 0x2B7,												STR_018E_LOWER_A_CORNER_OF_LAND},
{      WWT_PANEL,     7,    86,   107,    14,    35, 0x2B6,												STR_018F_RAISE_A_CORNER_OF_LAND},
{      WWT_PANEL,     7,   108,   129,    14,    35, 0x12B7,											STR_0329_PURCHASE_LAND_FOR_FUTURE},
{      WWT_LAST},
};


static const WindowDesc _air_toolbar_desc = {
	0x1FE, 0x16, 0x82, 0x24,
	WC_BUILD_TOOLBAR,0,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
	_air_toolbar_widgets,
	BuildAirToolbWndProc
};

void ShowBuildAirToolbar()
{
	DeleteWindowById(WC_BUILD_TOOLBAR, 0);
	AllocateWindowDescFront(&_air_toolbar_desc, 0);
}

static void BuildAirportPickerWndProc(Window *w, WindowEvent *e)
{
	switch(e->event) {
	case WE_PAINT: {
		int sel;

		if (WP(w,def_d).close)
			return;
		w->disabled_state = 0;

		sel = _selected_airport_type;
		if (!(_avail_aircraft & 1)) { w->disabled_state |= (1<<3); if (sel == AT_SMALL) sel = AT_LARGE; }
		if (!(_avail_aircraft & 2)) {	w->disabled_state |= (1<<4); if (sel == AT_LARGE) sel = AT_SMALL; }
		if (!(_avail_aircraft & 4)) {	w->disabled_state |= (1<<5); }
		_selected_airport_type = sel;
		w->click_state = ((1<<3) << sel) | ((1<<6) << _station_show_coverage);
		SetTileSelectSize(_airport_size_x[sel],_airport_size_y[sel]);
		if (_station_show_coverage)	SetTileSelectBigSize(-4, -4, 8, 8);

		DrawWindowWidgets(w);
		DrawStringCentered(74, 16, STR_305B_SIZE, 0);
		DrawStringCentered(74, 53, STR_3066_COVERAGE_AREA_HIGHLIGHT, 0);
		DrawStationCoverageAreaText(2, 77, (uint)-1);
		break;
	}

	case WE_CLICK: {
		switch(e->click.widget) {
		case 0:
			ResetObjectToPlace();
			break;
		case 3: case 4: case 5:
			_selected_airport_type = e->click.widget - 3;
			SndPlayFx(0x13);
			SetWindowDirty(w);
			break;
		case 6:
		case 7:
			_station_show_coverage = e->click.widget - 6;
			SndPlayFx(0x13);
			SetWindowDirty(w);
			break;
		}
	} break;

	case WE_MOUSELOOP: {
		if (WP(w,def_d).close) {
			DeleteWindow(w);
			return;
		}

		CheckRedrawStationCoverage(w);
	} break;
	}
}

static const Widget _build_airport_picker_widgets[] = {
{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,										STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,     7,    11,   147,     0,    13, STR_3001_AIRPORT_SELECTION,	STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,     7,     0,   147,    14,   107, 0x0,													0},
{WWT_NODISTXTBTN,    14,     2,    73,    27,    38, STR_3059_SMALL,							STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{WWT_NODISTXTBTN,    14,    74,   145,    27,    38, STR_305A_LARGE,							STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{WWT_NODISTXTBTN,    14,     2,   145,    39,    50, STR_306B_HELIPORT,						STR_3058_SELECT_SIZE_TYPE_OF_AIRPORT},
{   WWT_CLOSEBOX,    14,    14,    73,    62,    73, STR_02DB_OFF,								STR_3065_DON_T_HIGHLIGHT_COVERAGE},
{   WWT_CLOSEBOX,    14,    74,   133,    62,    73, STR_02DA_ON,									STR_3064_HIGHLIGHT_COVERAGE_AREA},
{      WWT_LAST},
};

static const WindowDesc _build_airport_desc = {
	-1, -1, 0x94, 0x6C,
	WC_BUILD_STATION,WC_BUILD_TOOLBAR,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
	_build_airport_picker_widgets,
	BuildAirportPickerWndProc
};

static void ShowBuildAirportPicker()
{
	AllocateWindowDesc(&_build_airport_desc);
}

void InitializeAirportGui()
{
	_selected_airport_type = AT_SMALL;
	_last_built_aircraft_depot_tile = 0;
}
